@@ -1,3 +1,4 @@
+import traceback
 from typing import AsyncIterable, Iterable
 
 import aiohttp
@@ -5,22 +6,21 @@ import requests
 from betterproto import Casing
 
 from maitai._config import config
-from maitai._maitai_client import MaiTaiClient
+from maitai._maitai_client import MaitaiClient
 from maitai._types import AsyncChunkQueue, ChunkQueue, EvaluateCallback, QueueIterable
 from maitai_gen.chat import ChatCompletionParams, ChatCompletionRequest
 from maitai_gen.inference import InferenceStreamResponse
 
 
-class Inference(MaiTaiClient):
+class Inference(MaitaiClient):
 
     def __init__(self):
         super().__init__()
 
     @classmethod
-    def infer(cls, session_id, reference_id, action_type, application_ref_name, completion_params: ChatCompletionParams, evaluate_callback: EvaluateCallback = None,
+    def infer(cls, session_id, reference_id, action_type, application_ref_name, completion_params: ChatCompletionParams, apply_corrections: bool, evaluation_enabled: bool, evaluate_callback: EvaluateCallback = None,
               timeout=None) -> Iterable[InferenceStreamResponse]:
-        apply_corrections = evaluate_callback is None
-        chat_request: ChatCompletionRequest = cls.create_inference_request(application_ref_name, session_id, reference_id, action_type, apply_corrections, completion_params)
+        chat_request: ChatCompletionRequest = cls.create_inference_request(application_ref_name, session_id, reference_id, action_type, apply_corrections, evaluation_enabled, completion_params)
         if evaluate_callback:
             q = ChunkQueue()
             cls.run_async(cls.send_inference_request_async(chat_request, chunk_queue=q, evaluation_callback=evaluate_callback))
@@ -29,16 +29,16 @@ class Inference(MaiTaiClient):
             return cls.send_inference_request(chat_request)
 
     @classmethod
-    async def infer_async(cls, session_id, reference_id, action_type, application_ref_name, completion_params: ChatCompletionParams, evaluate_callback: EvaluateCallback = None,
+    async def infer_async(cls, session_id, reference_id, action_type, application_ref_name, completion_params: ChatCompletionParams, apply_corrections: bool, evaluation_enabled: bool,
+                          evaluate_callback: EvaluateCallback = None,
                           timeout=None) -> AsyncIterable[InferenceStreamResponse]:
-        apply_corrections = evaluate_callback is None
-        chat_request: ChatCompletionRequest = cls.create_inference_request(application_ref_name, session_id, reference_id, action_type, apply_corrections, completion_params)
+        chat_request: ChatCompletionRequest = cls.create_inference_request(application_ref_name, session_id, reference_id, action_type, apply_corrections, evaluation_enabled, completion_params)
         q = AsyncChunkQueue()
         cls.run_async(cls.send_inference_request_async(chat_request, async_chunk_queue=q, evaluation_callback=evaluate_callback))
         return QueueIterable(q, timeout=timeout)
 
     @classmethod
-    def create_inference_request(cls, application_ref_name, session_id, reference_id, action_type, apply_corrections, completion_params: ChatCompletionParams):
+    def create_inference_request(cls, application_ref_name, session_id, reference_id, action_type, apply_corrections, evaluation_enabled, completion_params: ChatCompletionParams):
         infer_request: ChatCompletionRequest = ChatCompletionRequest()
         infer_request.application_ref_name = application_ref_name
         infer_request.reference_id = reference_id
@@ -46,6 +46,7 @@ class Inference(MaiTaiClient):
         infer_request.action_type = action_type
         infer_request.apply_corrections = apply_corrections
         infer_request.params = completion_params
+        infer_request.evaluation_enabled = evaluation_enabled
         return infer_request
 
     @classmethod
@@ -57,7 +58,6 @@ class Inference(MaiTaiClient):
                 'Content-Type': 'application/json',
                 'x-api-key': config.api_key
             }
-
             response = requests.post(url, headers=headers, data=chat_request.to_json(casing=Casing.SNAKE), verify=False, stream=True)
             if response.status_code != 200:
                 print(f"Failed to send inference request. Status code: {response.status_code}. Error: {response.text}")
@@ -93,7 +93,6 @@ class Inference(MaiTaiClient):
                     if async_chunk_queue:
                         await async_chunk_queue.put("done")
                     return
-                last_item = None
                 async for line in response.content:
                     if line:
                         inference_response: InferenceStreamResponse = InferenceStreamResponse().from_json(line)
@@ -101,11 +100,12 @@ class Inference(MaiTaiClient):
                             chunk_queue.put(inference_response)
                         if async_chunk_queue:
                             await async_chunk_queue.put(inference_response)
-                        last_item = inference_response
+                        if inference_response.evaluate_response and evaluation_callback:
+                            try:
+                                evaluation_callback(inference_response.evaluate_response)
+                            except:
+                                traceback.print_exc()
                 if chunk_queue:
                     chunk_queue.put("done")
                 if async_chunk_queue:
                     await async_chunk_queue.put("done")
-                if last_item is not None and last_item.evaluate_response:
-                    if evaluation_callback:
-                        evaluation_callback(last_item.evaluate_response)
